;**********************************************************************
; Poweramp_S1.asm 
; Control Software for Poweramp S1 project 
; 
; Controls On/Off, Power supply, Fan
; 
;
; Usage of Pins:
; GP5: relaiis ffor power supply
; GP4: On/Off LED & Temp Sensor power
; GP3: PWR-Switch (on/off)
; GP2: Fan
; GP1: Temp Sensor 2
; GP0: Temp Sensor 1
;
; Assuming INTOSC f=4MHz
;-------------------------------
;Stephan Asmus, Germany
;(c) stephan.asmus@t-online.de
;
; 9/2010
;-------------------------------
; 31.08.2010	Starting
; 01.09.2010 	Initial Version, working as expected
; 
; 
; 
;**********************************************************************
	LIST   P=PIC12F675

	#include <p12f675.inc>	; processor specific variable definitions


errorlevel  -302			;No bank select warnings

	__CONFIG (_CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_ON & _INTRC_OSC_NOCLKOUT & _CPD_OFF & _MCLRE_OFF)


;***** VARIABLE DEFINITIONS
	cblock 0x20			;define a series of variables.
		d1				; Register to hold Delayloop	
		d2				; dto.
		d3
		fanstat			;status of fan
						;Bits 0&1: Fan on/off
						;Bits 2&3: Overtemp on/off
		currtemp		;current temperature (equivalent)
		
        endc
; ***********Constants definitions
fan_on		equ d'79'	;equivalent to fan start-temp (50C)
fan_off		equ d'77'	;equivalennt to fan stop-temp (45C)
overtemp	equ d'99'	;equivalent to overtemp (100C)
overtemp_off equ d'95'	;equivalent to restart-temp if overtemp (90C)

#define PWR_out	GPIO,5	;output to power supply relais
#define	LED		GPIO,4	;on/off LED
#define	PWR		GPIO,3	;PWR switch
#define FAN		GPIO,2	;output to fan (1=on)


; **********************************************************************
; **********************************************************************
	ORG		0x000			; processor reset vector
	goto	init			; go to beginning of program


	ORG		0x004			; interrupt vector location
							;no interrupts

; ------------------------------------------------------------------------------
; START OF CODE to initialize the processor (Variables, I/O ...)
; ------------------------------------------------------------------------------
init
	bcf		STATUS,RP0		; Select Bank 0

	;initialise variables
	clrf	GPIO			; Init I/O-Port by clearing the outp. latches.

	bsf		STATUS, RP0		;Switch to BANK 1
	movlw	b'00001011'		;configure in/output (GP5,GP4,GP2: output;
	movwf	TRISIO			;...GP3,GP1,GP0: input)
	clrf	WPU				;no weak pullup
	movlw	b'00001000'		;intterrupt on change for GP3
	movwf	IOC
	movlw	b'00010011'		;configure AD to FOSC/8 and GP0+GP1 to analog
	movwf	ANSEL
	
	;calibrating the oscillator...
	call	3FFh 			;Get the cal value
	movwf	OSCCAL 			;Calibrate it

	bcf		STATUS, RP0		;Back to BANK 0

	movlw	b'00000001'		;configure AD: left just; Vref=VDD
	movwf	ADCON0			;Channel 0; AD=on

	bcf		INTCON,GIE		; no global interrupts
	bsf		INTCON,GPIE		;Interrupt on Change enabled
	bcf		INTCON,GPIF		;clear interrupt flag
	
	clrf	fanstat			;fan off
        
; ******************************************************************************
; MAIN, here we go...
; 
; ******************************************************************************
startup
	bsf		PWR_out			;power relais on
	bsf		LED				;LED on
	bcf		FAN				;Fan off
	clrf	fanstat			;status of fan cleared
	call 	delay1s			;and wait to settle
		
mainloop

	btfss	PWR				;pwr pressed?
	goto	do_sleep		;yes, then off & sleep
	
	bcf		ADCON0,CHS0		;select AD Channel 0
	bcf		ADCON0,CHS1
	call	get_ad			;sample temp
	movwf	currtemp		;save it
	movlw	fan_on			
	subwf	currtemp,w
	btfsc	STATUS,C		;skip if currtemp<fan_on
	bsf		fanstat,0		;set if currtemp > fan_on
	movlw	fan_off
	subwf	currtemp,w
	btfss	STATUS,C
	bcf		fanstat,0		;clear if currrtemp < fan_off
	movlw	overtemp
	subwf	currtemp,w
	btfsc	STATUS,C		;skip if currrtemp < overtemp
	bsf		fanstat,2		;set fanstat.2 =panic, shutdown
	movlw	overtemp_off
	subwf	currtemp,w
	btfss	STATUS,C
	bcf		fanstat,2		;clear if currrtemp < overtemp_off
	
	;now do the same for channel 1
	bsf		ADCON0,CHS0		;select AD Channel 1
	bcf		ADCON0,CHS1
	call	get_ad			;sample temp
	movwf	currtemp		;save it
	movlw	fan_on			
	subwf	currtemp,w
	btfsc	STATUS,C		;skip if currtemp<fan_on
	bsf		fanstat,1		;set if currtemp > fan_on
	movlw	fan_off
	subwf	currtemp,w
	btfss	STATUS,C
	bcf		fanstat,1		;clear if currrtemp < fan_off
	movlw	overtemp
	subwf	currtemp,w
	btfsc	STATUS,C		;skip if currrtemp < overtemp
	bsf		fanstat,3		;set fanstat.3 =panic, shutdown
	movlw	overtemp_off
	subwf	currtemp,w
	btfss	STATUS,C
	bcf		fanstat,3		;clear if currrtemp < overtemp_off

	;now check what to do with fanstat
	
	movf	fanstat,w			;fanstat=0?
	btfss	STATUS,Z
	bsf		FAN				;no, then set fan on
	movf	fanstat,w			;fanstat=0?
	btfsc	STATUS,Z
	bcf		FAN				;yes, then shut fan off
	
	movf	fanstat,w
	andlw	b'00001100'		;Bits 2 or 3 set (mask Bit 0 & 1)?
	btfss	STATUS,Z
	bcf		PWR_out			;yes, then shutdown powerstage
	movf	fanstat,w
	andlw	b'00001100'		;Bits 2 or 3 set?
	btfsc	STATUS,Z
	bsf		PWR_out			;no, then power powerstage
	
	movf	fanstat,w
	andlw	b'00001100'		;Bits 2 or 3 set?
	btfss	STATUS,Z
	call	blink_pwrled	;yes, then blink powerled
	
	
	goto 	mainloop		;start over

;-------------------------------------------------------------------------------
;blink_pwrled
;visual feedback to user that overtemp has ocurred
;-------------------------------------------------------------------------------
blink_pwrled

	bcf		LED
	call	delay100ms
	call	delay100ms
	bsf		LED
	call	delay100ms	
	call	delay100ms

	return

;-----------------------------------------------------------------
; Do Sleep
; Sets PIC to sleep after saving prefs
;-----------------------------------------------------------------
do_sleep
	bcf		PWR_out				;disable powerstage
	call	delay100ms
	call	delay100ms
	bcf		LED
	bcf		FAN
	btfss	PWR					;wait for key release
	goto	$-1
	call	delay1s
	bcf		INTCON,GPIF			;clear INT Flag
	movf	GPIO,w				;read GPIO to disable mismatch condition
	btfsc	INTCON,GPIF			;really clear?
	goto	$-3
	
	sleep						;now goto sleep
								;we sleep until PWR is pressed (GP3)
	
	nop
	movf	GPIO,w				;read GPIO to disable mismatch condition
	bcf		INTCON,GPIF			;clear INT Flag
	goto	startup				;and go on


;-------------------------------------------------------------------------------
;get_ad
;sample input on predefined port (port has to be selected before calling)
;-------------------------------------------------------------------------------
get_ad
	call	delay10us		;wait aquisation time

	bsf		ADCON0,1		;start conversion
	btfsc	ADCON0,1		;wait for completion
	goto	$-1
	movf	ADRESH,w		;get result
	return
; ------------------------------------------------------------------------------
; delay: several delay routines used in the program
; ------------------------------------------------------------------------------
delay1s
			;999990 cycles
	movlw	0x07
	movwf	d1
	movlw	0x2F
	movwf	d2
	movlw	0x03
	movwf	d3
delay1s_0
	decfsz	d1, f
	goto	$+2
	decfsz	d2, f
	goto	$+2
	decfsz	d3, f
	goto	delay1s_0
			;6 cycles
	goto	$+1
	goto	$+1
	goto	$+1
			;4 cycles (including call)
	return
;----
delay100ms
			;99993 cycles
	movlw	0x1E
	movwf	d1
	movlw	0x4F
	movwf	d2
delay_1
	decfsz	d1, f
	goto	$+2
	decfsz	d2, f
	goto	delay_1
			;3 cycles
	goto	$+1
	nop
			;4 cycles (including call)
	return
;----
delay10us
			;6 cycles
	goto	$+1
	goto	$+1
	goto	$+1

			;4 cycles (including call)
	return
; *****************************************************************
	END					;'end of program''
